@@ -6,27 +6,24 @@
 #
 # Claudio Perez
 #
-"""
-The glTF file format is supported by [COMSOL](https://www.comsol.com/blogs/how-to-export-and-share-your-3d-result-plots-as-gltf-files)
-"""
-
 """
 Some implementation notes:
+
 - glTF defines +Y as up.
 - glTF uses a right-handed coordinate system, that is, the cross product of +X and +Y yields +Z.
 - The front of a glTF asset faces +Z.
 
-
-- All angles are in radians.
-- Positive rotation is counterclockwise.
 - Rotations are given as quaternions stored as a tuple (x,y,z,w),
   where the w-component is the cosine of half of the rotation angle.
   For example, the quaternion [ 0.259, 0.0, 0.0, 0.966 ] describes a rotation
   about 30 degrees, around the x-axis.
 - The identity rotation is (0, 0, 0, 1.0)
+"""
 
 """
-import sys
+The glTF file format is supported by 
+[COMSOL](https://www.comsol.com/blogs/how-to-export-and-share-your-3d-result-plots-as-gltf-files)
+"""
 import itertools
 
 import numpy as np
@@ -34,7 +31,6 @@ import pygltflib
 from scipy.spatial.transform import Rotation
 
 import veux
-
 from .canvas import Canvas, Line, Mesh, Node
 from veux import utility
 from veux.config import NodeStyle, MeshStyle, LineStyle, DrawStyle
@@ -45,16 +41,12 @@ GLTF_T = {
     "uint16":  pygltflib.UNSIGNED_SHORT,
 }
 
-import numpy as np
-import pygltflib
-
 EYE3 = np.eye(3, dtype="float32")
 
 def _append_index(lst, item):
     lst.append(item)
     return len(lst) - 1
 
-
 class GltfLibCanvas(Canvas):
     vertical = 2
 
@@ -62,7 +54,7 @@ class GltfLibCanvas(Canvas):
         self.config = config
         self._data = {}
 
-        #                          x, y, z, scalar
+        #                 x, y, z, scalar
         self._rotation = [0, 0, 0, 1] #[-0.7071068, 0, 0, 0.7071068]
         # equivalent rotation matrix:
         self._rotation_matrix = np.eye(3) #np.array([[1,  0, 0],
@@ -163,6 +155,8 @@ class GltfLibCanvas(Canvas):
         
         self._arrows = {}
 
+        self._node_markers = {}
+
         #
         # load assets for steel material
         #
@@ -177,12 +171,10 @@ class GltfLibCanvas(Canvas):
             self.gltf.convert_images(pygltflib.ImageFormat.DATAURI)
 
     def _init_fixed_node(self, points, triangles, x=True, y=True, z=True):
-        # Reuse existing POSITION accessor indices assumed to be stored in self
-        points_access = self.position_accessor_index  # previously stored index for POSITION accessor
-        indices_access = self.indices_accessor_index  # previously stored index for indices accessor
+        points_access  = self.position_accessor_index
+        indices_access = self.indices_accessor_index
 
-
-        # Initialize an array for vertex colors, defaulting to a base color (e.g., gray)
+        # Initialize an array for vertex colors, defaulting to a base color
         vertex_colors = np.tile(np.array([0.5, 0.5, 0.5, 1.0], dtype=self.float_t), (len(points), 1))
 
         # For each triangle, check if the face lies on a plane fixed along given axes
@@ -191,7 +183,7 @@ class GltfLibCanvas(Canvas):
             # Extract vertices of the triangle
             v0, v1, v2 = points[tri[0]], points[tri[1]], points[tri[2]]
 
-            # Check if all vertices share the same coordinate on any fixed axis:
+            # Check if all vertices share the same coordinate on any fixed axis
             if x and (np.isclose(v0[0], v1[0]) and np.isclose(v1[0], v2[0])):
                 # Set vertices of this triangle to white
                 for vi in tri:
@@ -204,7 +196,6 @@ class GltfLibCanvas(Canvas):
                     vertex_colors[vi] = [1.0, 1.0, 1.0, 1.0]
 
 
-        # Push the color data to the buffer and create a new accessor for vertex colors
         color_accessor_index = len(self.gltf.accessors)
         self.gltf.accessors.append(
             pygltflib.Accessor(
@@ -251,11 +242,19 @@ class GltfLibCanvas(Canvas):
         return len(self.gltf.meshes) - 1
 
 
-    def _init_nodes(self, style: NodeStyle):
+    def _find_marker(self, style: NodeStyle):
+        key = (style.color, style.scale)
+        if key not in self._node_markers:
+            self._node_markers[key] = self._init_marker(style)
+
+        return self._node_markers[key]
+
+
+    def _init_marker(self, style: NodeStyle):
         #
         #
         #
-        index_t = self.index_t # "uint8"
+        index_t = self.index_t
         points = style.scale*np.array(
             [
                 [-1.0, -1.0,  1.0],
@@ -311,8 +310,85 @@ class GltfLibCanvas(Canvas):
         ])
 
 
-        indices_access = len(self.gltf.accessors)-2 # indices
-        points_access  = len(self.gltf.accessors)-1 # points
+        indices_access = len(self.gltf.accessors)-2
+        points_access  = len(self.gltf.accessors)-1
+        self.gltf.meshes.append(
+               pygltflib.Mesh(
+                 primitives=[
+                     pygltflib.Primitive(
+                         mode=pygltflib.TRIANGLES,
+                         attributes=pygltflib.Attributes(POSITION=points_access),
+                         material=self._get_material(style),
+                         indices=indices_access
+                     )
+                 ]
+               )
+        )
+
+        return len(self.gltf.meshes) - 1
+
+    def _init_nodes(self, style: NodeStyle):
+        #
+        #
+        #
+        index_t = self.index_t
+        points = style.scale*np.array(
+            [
+                [-1.0, -1.0,  1.0],
+                [ 1.0, -1.0,  1.0],
+                [-1.0,  1.0,  1.0],
+                [ 1.0,  1.0,  1.0],
+                [ 1.0, -1.0, -1.0],
+                [-1.0, -1.0, -1.0],
+                [ 1.0,  1.0, -1.0],
+                [-1.0,  1.0, -1.0],
+            ],
+            dtype=self.float_t,
+        )/10
+
+        triangles = np.array(
+            [
+                [0, 1, 2],
+                [3, 2, 1],
+                [1, 0, 4],
+                [5, 4, 0],
+                [3, 1, 6],
+                [4, 6, 1],
+                [2, 3, 7],
+                [6, 7, 3],
+                [0, 2, 5],
+                [7, 5, 2],
+                [5, 7, 4],
+                [6, 4, 7],
+            ],
+            dtype=index_t,
+        )
+        triangles_binary_blob = triangles.flatten().tobytes()
+
+        self.gltf.accessors.extend([
+            pygltflib.Accessor(
+                bufferView=self._push_data(triangles_binary_blob,
+                                           pygltflib.ELEMENT_ARRAY_BUFFER),
+                componentType=GLTF_T[index_t],
+                count=triangles.size,
+                type=pygltflib.SCALAR,
+                max=[int(triangles.max())],
+                min=[int(triangles.min())],
+            ),
+            pygltflib.Accessor(
+                bufferView=self._push_data(points.tobytes(),
+                                           pygltflib.ARRAY_BUFFER),
+                componentType=GLTF_T[self.float_t],
+                count=len(points),
+                type=pygltflib.VEC3,
+                max=points.max(axis=0).tolist(),
+                min=points.min(axis=0).tolist(),
+            )
+        ])
+
+
+        indices_access = len(self.gltf.accessors)-2
+        points_access  = len(self.gltf.accessors)-1
         self.gltf.meshes.append(
                pygltflib.Mesh(
                  primitives=[
@@ -339,6 +415,7 @@ class GltfLibCanvas(Canvas):
 
         color = style.color
 
+        rgb = None
         if (color, alpha) in self._color:
             return self._color[(color,alpha)]
 
@@ -351,7 +428,12 @@ class GltfLibCanvas(Canvas):
         elif isinstance(color, tuple):
             rgb = color
 
-        else:
+        elif isinstance(color, str):
+            for key,a in self._color:
+                if color == key:
+                    rgb = self.gltf.materials[self._color[(key,a)]].pbrMetallicRoughness.baseColorFactor[:3]
+
+        if rgb is None:
             raise TypeError("Unexpected type for color")
 
         # Store index for new material
@@ -361,7 +443,7 @@ class GltfLibCanvas(Canvas):
             pygltflib.Material(
                 name=str(color),
                 doubleSided=True,
-                alphaMode=pygltflib.MASK,
+                alphaMode=pygltflib.BLEND,
                 pbrMetallicRoughness=pygltflib.PbrMetallicRoughness(
                     baseColorFactor=[*rgb, alpha]
                 )
@@ -389,8 +471,10 @@ class GltfLibCanvas(Canvas):
     def plot_nodes(self, vertices, label = None, style=None, data=None, rotations=None, skin=False, **kwds):
         nodes = []
 
-        if not hasattr(self, "_node_mesh"):
-            self._init_nodes(style or NodeStyle())
+        # if not hasattr(self, "_node_mesh"):
+        #     self._init_nodes(style or NodeStyle())
+
+        marker = self._find_marker(style or NodeStyle())
 
         if rotations is None:
             rotations = itertools.repeat([0, 0, 0, 1.0])
@@ -403,7 +487,7 @@ class GltfLibCanvas(Canvas):
 
         for coord, rotation in zip(vertices, rotations):
             index = _append_index(self.gltf.nodes, pygltflib.Node(
-                    mesh=self._node_mesh,
+                    mesh=marker,
                     rotation=rotation,
                     translation=(self._rotation_matrix@coord).tolist(),
                 )
@@ -442,14 +526,7 @@ class GltfLibCanvas(Canvas):
         joint_node_to_index = {node: i for i, node in enumerate(joint_nodes)}
 
         points = np.zeros((len(joint_nodes), 3), dtype=self.float_t)
-        # points = np.zeros((1,3), dtype=self.float_t) 
-        # points = np.random.random((len(joint_nodes), 3))
-        # for i in range(len(joint_nodes)):
-        #     points[i] /= np.linalg.norm(points[i])*1e3
-        # points = points.astype(self.float_t)
 
-        # points = np.array([self._rotation_matrix.T@self.gltf.nodes[n].translation for n in joint_nodes], dtype=self.float_t)
-        # points = np.array(_,dtype=self.float_t) #np.array([self._rotation_matrix.T@n for n in _], dtype=self.float_t)
         ver_accessor = _append_index(self.gltf.accessors, pygltflib.Accessor(
                 bufferView=self._push_data(points.tobytes(), pygltflib.ARRAY_BUFFER),
                 componentType=GLTF_T[self.float_t],
@@ -460,7 +537,7 @@ class GltfLibCanvas(Canvas):
             )
         )
     
-        # Create Skeleton
+        # Create skeleton
         skeleton_root_node = pygltflib.Node(name="LineSkeleton", 
                                             children=joint_nodes)
         skeleton_root_idx = _append_index(gltf.nodes, skeleton_root_node)
@@ -483,7 +560,6 @@ class GltfLibCanvas(Canvas):
         # Add buffer view and accessor for the inverse bind matrices
         # Flatten inverse bind matrices for glTF format
         ibm_array = np.array(inverse_bind_matrices, dtype=self.float_t).reshape(-1, 16)
-        # ibm_array = np.array([ibm.flatten() for ibm in inverse_bind_matrices], dtype=self.float_t)
 
         ibm_accessor_idx = _append_index(gltf.accessors, pygltflib.Accessor(
             bufferView=self._push_data(ibm_array.tobytes(), target=None),
@@ -518,11 +594,9 @@ class GltfLibCanvas(Canvas):
                 [1.0, 0.0, 0.0, 0.0]
             ])
 
-        # index_array = np.array(indices, dtype=self.index_t)#.flatten()
-        index_array = np.array(lines, dtype=self.index_t).reshape(-1)#.flatten()
-        # index_array = np.zeros((len(lines),2), dtype=self.index_t).reshape(-1)
+        index_array = np.array(lines, dtype=self.index_t).reshape(-1)
 
-        # Add buffer view and accessor for the line indices
+        # Add buffer view and accessor
 
         index_accessor_idx = len(gltf.accessors)
         gltf.accessors.append(pygltflib.Accessor(
@@ -534,12 +608,11 @@ class GltfLibCanvas(Canvas):
             max=[int(index_array.max())]
         ))
 
-        # Create the line mesh with skinning attributes
-        joints_0_array  = np.array(joints_0, dtype=self.index_t)
+        # Create the line mesh
 
         joints_0_accessor_idx = len(gltf.accessors)
         gltf.accessors.append(pygltflib.Accessor(
-            bufferView=self._push_data(joints_0_array.tobytes(), pygltflib.ARRAY_BUFFER),
+            bufferView=self._push_data(np.array(joints_0, dtype=self.index_t).tobytes(), pygltflib.ARRAY_BUFFER),
             componentType=GLTF_T[self.index_t],
             count=len(joints_0),
             type="VEC4"
@@ -599,7 +672,208 @@ class GltfLibCanvas(Canvas):
         self._data[key] = {"access_index": len(self.gltf.accessors)-1}
 
 
-    def plot_lines(self, vertices, indices=None, style: LineStyle=None, **kwds):
+    def _make_line_strip(self, indices, vertices, points_access, material):
+        
+        # `n` adjusts indices by the number of nan rows that were removed so far
+        if not isinstance(vertices, (int, str)):
+            n  = sum(np.isnan(vertices[:indices[0],0]))
+        else:
+            n = 0
+
+        indices_array = indices - np.dtype(self.index_t).type(n)
+
+        indices_binary_blob = indices_array.tobytes()
+
+        if len(indices_array) <= 1:
+            import warnings
+            warnings.warn(indices_array)
+            return None
+
+        self.gltf.accessors.extend([
+            pygltflib.Accessor(
+                bufferView=self._push_data(indices_binary_blob,
+                                            pygltflib.ELEMENT_ARRAY_BUFFER),
+                componentType=GLTF_T[self.index_t],
+                count=indices_array.size,
+                type=pygltflib.SCALAR,
+                max=[int(indices_array.max())],
+                min=[int(indices_array.min())],
+            )
+        ])
+        self.gltf.meshes.append(
+                pygltflib.Mesh(
+                    primitives=[
+                        pygltflib.Primitive(
+                            mode=pygltflib.LINE_STRIP,
+                            attributes=pygltflib.Attributes(POSITION=points_access),
+                            material=material,
+                            # most recently added accessor
+                            indices=len(self.gltf.accessors)-1,
+                            # TODO: use this mechanism to add annotation data
+                            extras={},
+                        )
+                    ]
+                )
+        )
+
+        self.gltf.nodes.append(pygltflib.Node(
+                mesh=len(self.gltf.meshes)-1,
+                rotation=self._rotation
+            )
+        )
+
+        self.gltf.scenes[0].nodes.append(
+            len(self.gltf.nodes)-1
+        )
+
+        return Line(id=len(self.gltf.nodes)-1)
+
+    def plot_lines(self, vertices, indices=None, style: LineStyle | None = None, **node_kwds):
+        """
+        Add a batch of disconnected line-segments to the scene as a *single*
+        glTF primitive (mode = LINES).
+
+        Parameters
+        ----------
+        vertices : array_like, int, or str
+            • (N, 3) array – NaNs may delimit poly-lines  
+            • accessor index/name – reuse an existing POSITION accessor
+        indices : sequence[array_like] | None
+            If given, each item is a vertex-index list describing one poly-line.
+            Consecutive pairs are turned into independent segments.  If omitted
+            and ``vertices`` is an array with NaNs, the index list is generated
+            automatically.
+        style   : LineStyle, optional
+            Passed to ``_get_material``.
+        **node_kwds
+            Extra kwargs for the created glTF ``Node`` (e.g. “name”, “matrix”, ...).
+
+        Returns
+        -------
+        int
+            Index of the new ``Node`` in ``self.gltf.nodes``.
+        """
+        #
+        # 1.  Material
+        #
+        material_idx = self._get_material(style or LineStyle())
+
+        # ------------------------------------------------------------------
+        # 2.  POSITION accessor
+        # ------------------------------------------------------------------
+        if isinstance(vertices, (int, str)):
+            # Re-use an accessor that caller has already stored.
+            points_access = self.get_data(vertices)["access_index"]
+            n_vertices = self.gltf.accessors[points_access].count
+            # When the caller passes indices, we trust them to
+            # reference this existing accessor correctly.
+            verts_map = None          # identity
+        else:
+            return self.plot_lines_old(vertices, indices, style, **node_kwds)
+            vertices = np.asarray(vertices, dtype=self.float_t)
+            if vertices.ndim != 2 or vertices.shape[1] != 3:
+                raise ValueError("vertices must be (N,3)")
+
+            # Strip NaN rows – these are *delimiters*, never emitted.
+            finite_mask = np.isfinite(vertices[:, 0])
+            points = vertices[finite_mask]
+            if points.size == 0:
+                return None  # nothing to draw
+
+            buf_view = self._push_data(points.tobytes(), pygltflib.ARRAY_BUFFER)
+            self.gltf.accessors.append(
+                pygltflib.Accessor(
+                    bufferView=buf_view,
+                    componentType=GLTF_T[self.float_t],
+                    count=len(points),
+                    type=pygltflib.VEC3,
+                    min=points.min(axis=0).tolist(),
+                    max=points.max(axis=0).tolist(),
+                )
+            )
+            points_access = len(self.gltf.accessors) - 1
+            n_vertices = len(points)
+
+            # Map original row index to new compact index
+            verts_map = np.full(len(vertices), -1, dtype=self.index_t)
+            verts_map[finite_mask] = np.arange(n_vertices, dtype=self.index_t)
+
+        # ------------------------------------------------------------------
+        # 3.  Build flat SCALAR index list (pairs)
+        # ------------------------------------------------------------------
+        seg_idx = []
+        if indices is not None:
+            for poly in indices:
+                poly = np.asarray(poly, dtype=self.index_t)
+                if verts_map is not None:
+                    poly = verts_map[poly]          # translate to compact space
+                if len(poly) < 2:
+                    continue
+                # consecutive pairs
+                for i in range(len(poly) - 1):
+                    a, b = poly[i], poly[i + 1]
+                    if a != b:                      # skip degenerate
+                        seg_idx.extend((a, b))
+        else:
+            if verts_map is None:
+                raise ValueError("Automatic index generation needs vertices array.")
+            current = []
+            for src_idx, compact_idx in enumerate(verts_map):
+                if compact_idx == -1:               # NaN delimiter
+                    current.clear()
+                    continue
+                current.append(compact_idx)
+                if len(current) >= 2:
+                    # last two points form a segment
+                    seg_idx.extend(current[-2:])
+        if not seg_idx:
+            return None
+
+        seg_idx = np.asarray(seg_idx, dtype=self.index_t)
+        if seg_idx.max() >= n_vertices:
+            raise ValueError("Index out of range for POSITION accessor.")
+
+
+        # ------------------------------------------------------------------
+        # Create lines
+        # ------------------------------------------------------------------
+
+        #
+        # INDICES accessor
+        # 
+        idx_view = self._push_data(seg_idx.tobytes(), pygltflib.ELEMENT_ARRAY_BUFFER)
+        self.gltf.accessors.append(
+            pygltflib.Accessor(
+                bufferView=idx_view,
+                componentType=GLTF_T[self.index_t],
+                count=seg_idx.size,
+                type=pygltflib.SCALAR,
+                min=[int(seg_idx.min())],
+                max=[int(seg_idx.max())],
+            )
+        )
+        idx_acc = len(self.gltf.accessors) - 1
+
+        #
+        # Primitive, Mesh, and Node
+        #
+        primitive = pygltflib.Primitive(
+            attributes={"POSITION": points_access},
+            indices=idx_acc,
+            mode=pygltflib.LINES,
+            material=material_idx,
+        )
+        self.gltf.meshes.append(pygltflib.Mesh(primitives=[primitive]))
+        mesh_idx = len(self.gltf.meshes) - 1
+
+        self.gltf.nodes.append(pygltflib.Node(mesh=mesh_idx, **node_kwds))
+        node_idx = len(self.gltf.nodes) - 1
+
+        self.gltf.scenes[0].nodes.append(node_idx)
+
+        return Line(id=node_idx)
+
+    def plot_lines_old(self, vertices, indices=None, style: LineStyle=None, **kwds):
 
         lines = []
         material = self._get_material(style or LineStyle())
@@ -631,6 +905,7 @@ class GltfLibCanvas(Canvas):
             )
             points_access = len(self.gltf.accessors) - 1
 
+
         if indices is None:
             # Get indices by splitting at nans
             indices_ = utility.split(np.arange(len(vertices), dtype=self.index_t), np.nan, vertices[:,0])
@@ -638,61 +913,9 @@ class GltfLibCanvas(Canvas):
             indices_ = list(map(lambda x: np.array(x, dtype=self.index_t), indices))
 
         for indices in indices_:
-            # `n` adjusts indices by the number of nan rows that were removed so far
-            if not isinstance(vertices, (int, str)):
-                n  = sum(np.isnan(vertices[:indices[0],0]))
-            else:
-                n = 0
-
-            indices_array = indices - np.dtype(self.index_t).type(n)
-
-            indices_binary_blob = indices_array.tobytes()
-
-            if len(indices_array) <= 1:
-                import warnings
-                warnings.warn(indices_array)
+            if (line := self._make_line_strip(indices, vertices, points_access, material)) is None:
                 continue
-
-            self.gltf.accessors.extend([
-                pygltflib.Accessor(
-                    bufferView=self._push_data(indices_binary_blob,
-                                               pygltflib.ELEMENT_ARRAY_BUFFER),
-                    componentType=GLTF_T[self.index_t],
-                    count=indices_array.size,
-                    type=pygltflib.SCALAR,
-                    max=[int(indices_array.max())],
-                    min=[int(indices_array.min())],
-                )
-            ])
-            self.gltf.meshes.append(
-                   pygltflib.Mesh(
-                     primitives=[
-                         pygltflib.Primitive(
-                             mode=pygltflib.LINE_STRIP,
-                             attributes=pygltflib.Attributes(POSITION=points_access),
-                             material=material,
-                             # most recently added accessor
-                             indices=len(self.gltf.accessors)-1,
-                             # TODO: use this mechanism to add annotation data
-                             extras={},
-                         )
-                     ]
-                   )
-            )
-
-            self.gltf.nodes.append(pygltflib.Node(
-                    mesh=len(self.gltf.meshes)-1,
-                    rotation=self._rotation
-                )
-            )
-
-            self.gltf.scenes[0].nodes.append(
-                len(self.gltf.nodes)-1
-            )
-
-            lines.append(Line(
-                id=len(self.gltf.nodes)-1
-            ))
+            lines.append(line)
 
         return lines
 
@@ -733,10 +956,10 @@ class GltfLibCanvas(Canvas):
             # Check if they are already the same (no rotation) or exactly opposite
             dot_val = np.dot(x_axis, v_norm)
             if np.isclose(dot_val, 1.0):
-                # Same direction -> identity quaternion
+                # Same direction, set to identity quaternion
                 return Rotation.from_quat([0.0, 0.0, 0.0, 1.0])
             elif np.isclose(dot_val, -1.0):
-                # Opposite direction -> 180 deg rotation about any axis perpendicular to x
+                # Opposite direction, 180 deg rotation about any axis perpendicular to x
                 # e.g., about [0,1,0]
                 return Rotation.from_rotvec(np.pi * np.array([0, 1, 0]))
 
@@ -905,7 +1128,7 @@ class GltfLibCanvas(Canvas):
 
 
     def plot_mesh_field(self, mesh_handle, field,
-                        colormap="twilight", #"viridis", 
+                        colormap="rainbow4", # "cet_CET_R1",# "cet_CET_D13"#"twilight", #"viridis", 
                         vmin=None, vmax=None,
                         **kwds) -> tuple:
         """
@@ -921,15 +1144,9 @@ class GltfLibCanvas(Canvas):
         :param kwds: other keywords
         """
 
-        # Step 1. Determine color array from 'field'
-        #         For demonstration, let's assume you have a helper
-        #         that returns Nx4 RGBA in [0,1].
+        # Determine color array from field
         color_array = self._map_field_to_colors(field, colormap, vmin, vmax)
 
-        # Step 2. Plot the mesh (re-use plot_mesh) but we'll also attach colors
-        # mesh_handle = self.plot_mesh(vertices, triangles, style=style, **kwds)
-
-        # Step 3. Add color data as a separate accessor in the just-created mesh
         color_array = color_array.astype(self.float_t)
         self.gltf.accessors.extend([
             pygltflib.Accessor(
@@ -949,29 +1166,49 @@ class GltfLibCanvas(Canvas):
 
         return mesh_handle
 
-
     def _map_field_to_colors(self, field, 
-                             colormap="twilight", #"viridis", 
+                             colormap="cet_CET_D13", #"rainbow4", #"viridis", 
                              vmin=None, vmax=None):
         """
-        Convert 1D array of scalar values into Nx4 RGBA array in [0,1].
         """
-        import matplotlib
-        import matplotlib.cm
+        try:
+            import colorcet as cc
+            field = np.asarray(field, dtype=np.float64)
 
-        field = np.array(field, dtype=np.float64)
-        if vmin is None:
-            vmin = field.min()
-        if vmax is None:
-            vmax = field.max()
+            # Normalisation
+            if vmin is None:
+                vmin = np.nanmin(field)
+            if vmax is None:
+                vmax = np.nanmax(field)
 
-        norm = matplotlib.colors.Normalize(vmin=vmin, vmax=vmax, clip=True)
-        cmap = matplotlib.cm.get_cmap(colormap)
-        
-        # shape: (N, 4)
-        colors = cmap(norm(field))
-        # ensure Nx4
-        return colors
+            # avoid divide-by-zero
+            span = vmax - vmin or 1.0
+            t = np.clip((field - vmin) / span, 0.0, 1.0)
+
+
+            try:
+                cmap = cc.cm[colormap]
+            except KeyError as e:
+                raise ValueError(f"Unknown ColorCET colormap '{colormap}'.") from e
+
+            colors = cmap(t)
+            return colors
+
+        except:
+            import matplotlib
+            import matplotlib.cm
+
+            field = np.array(field, dtype=np.float64)
+            if vmin is None:
+                vmin = field.min()
+            if vmax is None:
+                vmax = field.max()
+
+            norm = matplotlib.colors.Normalize(vmin=vmin, vmax=vmax, clip=True)
+            cmap = matplotlib.cm.get_cmap(colormap)
+
+            colors = cmap(norm(field))
+            return colors
 
 
     def to_glb(self)->bytes:
