@@ -9,12 +9,11 @@
 import warnings
 import numpy as np
 Array = np.ndarray
-from scipy.linalg import block_diag
 
 import shps.rotor as so3
 from veux.model  import Model,FrameModel
 from veux.state  import State, BasicState
-from veux.config import Config, LineStyle, NodeStyle
+from veux.config import Config, LineStyle, NodeStyle, MeshStyle
 from ._section import SectionGeometry
 
 class FrameArtist:
@@ -77,6 +76,7 @@ class FrameArtist:
 
         # Define transformation from DOFs in `state` variables to plot
         # coordinates
+        from scipy.linalg import block_diag
         if model.ndf == 1 and model.ndm == 2:
             self.dofs2plot = R@np.array(((0,),
                                          (0,),
@@ -109,6 +109,14 @@ class FrameArtist:
 
         self.displ_states = {}
 
+    def plot_scatter(self, vertices, label=None, style=None, color=None, scale=1.0):
+        """
+        Plot a scatter of vertices in the canvas.
+        """
+        if style is None:
+            style = NodeStyle(color=color or "#808080", scale=scale)
+
+        self.canvas.plot_nodes([self._plot_rotation@x for x in vertices], style=style)
 
     def _config_sketch(self,  sketch):
         strokes = {"outline", "surface", "axes", "contour", "marker", "info"}
@@ -211,7 +219,160 @@ class FrameArtist:
     def draw_skeleton(self, skins, state=None, config=None, scale=1.0):
         pass
 
-    def draw_outlines(self, state=None, config=None, scale=1.0, skip=None):
+    def _draw_diagram_x(self, field, scale=1.0, config=None):
+        pass
+
+    def draw_diagrams(self, field,  scale=1.0, config=None, color=None):
+        """
+        Draws diagram fields on line-like elements (frame and truss) as vertical extrusions.
+
+        Parameters
+        ----------
+        field : callable
+            A function taking (element_tag: int, x: float) and returning a scalar field value.
+            x ∈ [0, 1] is the normalized local coordinate along the element.
+        scale : float, optional
+            Scale factor for the diagram magnitudes (applied vertically). Default is 1.0.
+        config : dict, optional
+            Optional style configuration. Uses 'frame' entry if provided.
+        """
+
+        Ra = self._plot_rotation
+        model = self.model
+
+        if config is None:
+            from veux.config import SketchConfig
+            config = {type: conf["surface"] for type, conf in SketchConfig().items() if "surface" in conf}
+            config.setdefault("frame", {})["show"] = True
+
+        if not config.get("frame", {}).get("show", True):
+            return
+
+        offset = np.zeros(3)
+        offset[2] = 1.0  # vertical direction
+
+        nodes = []
+        triangles = []
+        node_index = 0
+        Rm = np.array([[0, 0, 0],
+                       [0, 0, 1],
+                       [0,-1, 0]])
+
+        if hasattr(field, "x"):
+            xe = field.x
+            ye = field.y
+        else:
+            xe = None 
+            ye = None
+
+        for tag in model.iter_cell_tags():
+            if not model.cell_matches(tag, "frame") and not model.cell_matches(tag, "truss"):
+                continue
+
+            X = model.cell_position(tag)
+            X = np.array([Ra @ x for x in X])  # rotate to view coordinates
+            Re = model.frame_orientation(tag)
+            if Re is None:
+                continue
+            
+            N = 0
+            for i,(xi, yi) in enumerate(zip(*field(tag))):
+                L = np.linalg.norm(X[0] - X[1])
+                x_ = (1 - xi) * X[0] + xi * X[1]
+                y_ = np.array(yi)*scale
+                nodes.append(x_)
+                nodes.append(x_ + Ra@Re.T@Rm@Re@Ra.T@y_)
+                N += 1
+            N -= 1
+
+
+            # Add two triangles per segment
+            for i in range(N):
+                n0 = node_index + 2 * i
+                n1 = node_index + 2 * i + 1
+                n2 = node_index + 2 * i + 3
+                n3 = node_index + 2 * i + 2
+
+                triangles.append([n0, n1, n2])
+                triangles.append([n0, n2, n3])
+
+            node_index += 2 * (N + 1)
+
+        if len(triangles) > 0:
+            self.canvas.plot_mesh(np.array(nodes), 
+                                  np.array(triangles), 
+                                  style=MeshStyle(color=(color or "red"),
+                                                  alpha=0.5))
+        
+
+    def _draw_frame_lines(self, state=None, config=None, scale=1.0, style=None, skip=None):
+        """
+        """
+
+        Ra = self._plot_rotation
+        model = self.model
+
+        if config is None:
+            from veux.config import SketchConfig
+            config = {type: conf["outline"] for type, conf in SketchConfig().items() if "outline" in conf}
+            if style is not None:
+                config["frame"]["style"] = style
+                config["plane"]["style"] = style
+                config["solid"]["style"] = style
+
+        if skip is not None:
+            for type in skip:
+                config[type]["show"] = False
+
+        if state is not None:
+            state = self.model.wrap_state(state, scale=scale, transform=self.dofs2plot)
+            # Because "state" is given as opposed to position/displacement, we assume
+            # linearized rotations for now.
+            config["frame"]["basis"] = "Hermite"
+
+        N = 20 if state is not None and config["frame"]["basis"] is not None else 2
+        do_frames = False
+        ne = sum(1 for tag in model.iter_cell_tags() 
+                 if model.cell_matches(tag, "frame") or model.cell_matches(tag, "truss"))
+        frames = np.zeros((ne*(N+1),3))
+        frames.fill(np.nan)
+
+        j = 0 # frame element counter
+        for tag in model.iter_cell_tags():
+            if model.cell_matches(tag, "frame"): # or model.cell_matches(tag, "truss"):
+                if not config["frame"]["show"]:
+                    continue
+
+                if config["frame"]["basis"] is None or not model.cell_matches(tag, "frame"):
+                    # Draw a straight line between nodes
+                    do_frames = True
+                    frames[(N+1)*j:(N+1)*j+N,:] = np.linspace(*[
+                        Ra@xn for xn in model.cell_position(tag, state)[[0,-1],:]
+                    ], N)
+
+                else:
+                    do_frames = True
+                    Q = model.frame_orientation(tag)
+                    X = model.cell_position(tag)
+                    u = [xi - Xi for Xi, xi in zip(X, model.cell_position(tag, state=state))]
+                    R = [model.node_rotation(node, state=state) for node in model.cell_nodes(tag)]
+
+
+                    frames[(N+1)*j:(N+1)*j+N,:] = [Ra@v for v in  
+                                                    _hermite_cubic(X,
+                                                                 Q=Q,
+                                                                 u = [Q@u[0],          Q@u[1]],
+                                                                 v = [Q@so3.log(R[0]), Q@so3.log(R[1])],
+                                                                 npoints=N).T]
+                j += 1
+
+
+        if do_frames and config["frame"]["show"]:
+            self.canvas.plot_lines(frames[:,:self.ndm], style=config["frame"]["style"])
+
+
+
+    def draw_outlines_(self, state=None, config=None, scale=1.0, style=None, skip=None):
         """
         Draw the outlines of the model elements such as frames, planes, and solids.
         Interpolate if possible. This is expensive.
@@ -232,6 +393,10 @@ class FrameArtist:
         if config is None:
             from veux.config import SketchConfig
             config = {type: conf["outline"] for type, conf in SketchConfig().items() if "outline" in conf}
+            if style is not None:
+                config["frame"]["style"] = style
+                config["plane"]["style"] = style
+                config["solid"]["style"] = style
 
         if skip is not None:
             for type in skip:
@@ -243,47 +408,15 @@ class FrameArtist:
             # linearized rotations for now.
             config["frame"]["basis"] = "Hermite"
 
-        N = 20 if state is not None and config["frame"]["basis"] is not None else 2
-        do_frames = False
-        ne = sum(1 for tag in model.iter_cell_tags() 
-                 if model.cell_matches(tag, "frame") or model.cell_matches(tag, "truss"))
-        frames = np.zeros((ne*(N+1),3))
-        frames.fill(np.nan)
+        self._draw_frame_lines(state=state, config=config, scale=scale, style=style, skip=skip)
+
 
         quadrs = []
         trians = []
         solids = []
-
-        j = 0 # frame element counter
         for tag in model.iter_cell_tags():
-            if model.cell_matches(tag, "frame") or model.cell_matches(tag, "truss"):
-                if not config["frame"]["show"]:
-                    continue
 
-                if config["frame"]["basis"] is None or not model.cell_matches(tag, "frame"):
-                    # Draw a straight line between nodes
-                    do_frames = True
-                    frames[(N+1)*j:(N+1)*j+N,:] = np.linspace(*[
-                        Ra@xn for xn in model.cell_position(tag, state)[[0,-1],:]
-                    ], N)
-
-                else:
-                    Q = model.frame_orientation(tag)
-                    X = model.cell_position(tag)
-                    u = [xi - Xi for Xi, xi in zip(X, model.cell_position(tag, state=state))]
-                    R = [model.node_rotation(node, state=state) for node in model.cell_nodes(tag)]
-
-
-                    do_frames = True
-                    frames[(N+1)*j:(N+1)*j+N,:] = [Ra@v for v in  
-                                                    _hermite_cubic(X,
-                                                                 Q=Q,
-                                                                 u = [Q@u[0],          Q@u[1]],
-                                                                 v = [Q@so3.log(R[0]), Q@so3.log(R[1])],
-                                                                 npoints=N).T]
-                j += 1
-
-            elif model.cell_matches(tag, "plane") and config["plane"]["show"]:
+            if model.cell_matches(tag, "plane") and config["plane"]["show"]:
                 idx = model.cell_exterior(tag)
                 if len(idx) == 4:
                     quadrs.append([*idx, idx[0]])
@@ -295,11 +428,12 @@ class FrameArtist:
                 idx = model.cell_exterior(tag)
                 solids.append(idx)
 
-        if do_frames and config["frame"]["show"]:
-            self.canvas.plot_lines(frames[:,:self.ndm], style=config["frame"]["style"])
-
         nodes = np.array([Ra@x for x in model.node_position(state=state)])
 
+        # Add a small offset to the lines to avoid z-fighting with any surfaces
+        if self.model.ndm == 2:
+            nodes += Ra@[0, 0, 0.01]
+
         if len(quadrs) > 0 and config["plane"]["show"]:
             self.canvas.plot_lines(nodes, indices=np.array(quadrs),
                                    style=config["plane"]["style"])
@@ -312,12 +446,99 @@ class FrameArtist:
             self.canvas.plot_lines(nodes, indices=np.array(solids),
                                           style=config["solid"]["style"])
 
+    def draw_outlines(self, state=None, config=None, scale=1.0, style=None, skip=None):
+        """
+        Draw the outlines of the model elements such as frames, planes, and solids.
+        Interpolate if possible.
+
+        Parameters
+        ----------
+        state : dict, np.ndarray, callable, optional
+            The state of the model, see :ref:`State`. Default is None.
+        config : dict, optional
+            Configuration dictionary for drawing outlines. If None, a default configuration is loaded from SketchConfig. Default is None.
+        scale : float, optional
+            Scaling factor for the model. Default is 1.0.
+        """
+
+        Ra = self._plot_rotation
+        model = self.model
+
+        if config is None:
+            from veux.config import SketchConfig
+            config = {type: conf["outline"] for type, conf in SketchConfig().items() if "outline" in conf}
+            if style is not None:
+                config["frame"]["style"] = style
+                config["plane"]["style"] = style
+                config["solid"]["style"] = style
+
+        if skip is not None:
+            for type in skip:
+                config[type]["show"] = False
+
+        if state is not None:
+            state = self.model.wrap_state(state, scale=scale, transform=self.dofs2plot)
+            # Because "state" is given as opposed to position/displacement, we assume
+            # linearized rotations for now.
+            config["frame"]["basis"] = "Hermite"
+
+        self._draw_frame_lines(state=state, config=config, scale=scale, style=style, skip=skip)
+
+        lines  = []
+        quadrs = []
+        trians = []
+        solids = []
+        for tag in model.iter_cell_tags():
+
+            if model.cell_matches(tag, "plane") and config["plane"]["show"]:
+                idx = model.cell_exterior(tag)
+                if len(idx) == 4:
+                    quadrs.append([*idx, idx[0]])
+                elif len(idx) == 3:
+                    trians.append([*idx, idx[0]])
+
+            elif model.cell_matches(tag, "solid") and config["solid"]["show"]:
+                # TODO: get cell faces
+                idx = model.cell_exterior(tag)
+                if idx:
+                    solids.append(idx)
+            
+            elif model.cell_matches(tag, "truss"):
+                lines.append(model.cell_exterior(tag))
+
+
+        nodes = np.array([Ra@x for x in model.node_position(state=state)])
+
+        # Add a small offset to the lines to avoid z-fighting with any surfaces
+        if self.model.ndm == 2:
+            nodes += Ra@[0, 0, 0.01]
+        
+        self.canvas.set_data(nodes, key="node_vertices")
+
+
+        if len(quadrs) > 0 and config["plane"]["show"]:
+            self.canvas.plot_lines("node_vertices", indices=np.array(quadrs),
+                                   style=config["plane"]["style"])
+
+        if len(trians) > 0 and config["plane"]["show"]:
+            self.canvas.plot_lines("node_vertices", indices=np.array(trians),
+                                   style=config["plane"]["style"])
+
+        if len(solids) > 0 and config["solid"]["show"]:
+            self.canvas.plot_lines("node_vertices", indices=np.array(solids),
+                                          style=config["solid"]["style"])
+
+        if len(lines) > 0:
+            self.canvas.plot_lines("node_vertices", indices=np.array(lines),
+                                    style=config["frame"]["style"])
+
+
     def draw_sections(self,
                       state=None, rotation=None, position=None, scale=1.0,
-                      config=None):
+                      mesh_style=None, config=None):
         """
         Draw beam elements with extruded cross-sections. By default, cross-sectional
-        information is extracted from the model by various means. 
+        information is extracted from the model by various means.
         For `fiber <https://xara.so/user/manual/section/ShearFiber.html>`__ sections the outline is automatically computed
         using exterior fibers.
 
@@ -332,7 +553,6 @@ class FrameArtist:
         scale : float, optional
             Scale factor for the state transformation. Default is 1.0.
 
-
         """
         model = self.model
         Ra = self._plot_rotation
@@ -348,6 +568,9 @@ class FrameArtist:
             from veux.config import SketchConfig
             config = {type: conf["surface"] for type, conf in SketchConfig().items() if "surface" in conf}
 
+        if mesh_style is not None:
+            config["frame"]["style"] = mesh_style
+
         # Draw extruded frames
         from veux.frame import extrude
         extrude.draw_extrusions3(model,
@@ -356,9 +579,21 @@ class FrameArtist:
                                 Ra=Ra,
                                 config=config["frame"])
 
+
+    def draw_samples(self, style=None):
+        if style is None :
+            style = NodeStyle(color="blue")
+
+        for tag in self.model.iter_cell_tags():
+            for x, w in self.model.cell_quadrature(tag):
+                # style.scale = np.sqrt(w)*10
+                self.canvas.plot_nodes([x], style=style)
+
+
     def draw_surfaces(self,
                       state=None, field=None,  # States
-                      config=None, scale=1.0): # Drawing
+                      normal=None,
+                      config=None, scale=1.0, style=None): # Drawing
         """
         Draws surfaces on the canvas based on the provided state, field, layer, and configuration.
 
@@ -384,6 +619,9 @@ class FrameArtist:
             from veux.config import SketchConfig
             config = {type: conf["surface"] for type, conf in SketchConfig().items() if "surface" in conf}
             config["frame"]["show"] = True
+            if style is not None:
+                config["frame"]["style"] = style
+                config["plane"]["style"] = style
 
         # Draw extruded frames
         if "frame" in config and config["frame"]["show"]:
@@ -409,6 +647,8 @@ class FrameArtist:
             if field is not None:
                 if isinstance(field, dict):
                     field = np.array([field[node] for node in model.iter_node_tags()])
+                elif callable(field):
+                    field = np.array([field(tag) for tag in model.iter_node_tags()])
                 self.canvas.plot_mesh_field(mesh, field=field)
 
         return
@@ -416,11 +656,14 @@ class FrameArtist:
     def draw_nodes(self,
                    state=None,
                    data=None, label=None, config=None, size=None, scale=1.0):
+        
         R = self._plot_rotation 
 
         from veux.config import SketchConfig
         if config is None:
-            config = {type: conf["marker"] for type, conf in SketchConfig().items() if "marker" in conf}
+            config = {type: conf["marker"] 
+                      for type, conf in SketchConfig().items() if "marker" in conf}
+
         if size is not None:
             config["node"]["style"].scale = size
 
@@ -435,6 +678,7 @@ class FrameArtist:
             rotations = None
 
         coord = np.array([R@self.model.node_position(tag, state=state) for tag in self.model.iter_node_tags()])
+        
         self.canvas.plot_nodes(coord[:,:self.ndm], label=label,
                                rotations=rotations,
                                style=config["node"]["style"])
@@ -478,12 +722,10 @@ class FrameArtist:
                                  np.array([Ra@v for v in uvw.reshape(ne*3,3)]),
                                  extrude=extrude)
 
+
     def draw_origin(self, **kwds):
         xyz = np.zeros((3,3))
         uvw = self._plot_rotation.T*kwds.get("scale", 1.0)
-        off = [[0, -kwds.get("scale", 1.0)/2, 0],
-               [0]*3,
-               [0]*3]
 
         self.canvas.plot_vectors(xyz, uvw, **kwds)
 
@@ -520,7 +762,11 @@ class FrameArtist:
 
     def _repr_html_(self):
         from veux.viewer import Viewer
-        viewer = Viewer(self,hosted=False,standalone=False)
+        viewer = Viewer(self,
+                        size=(800, 600),
+                        hosted=False,
+                        show_quit=False,
+                        standalone=False)
         html = viewer.get_html()
         return html
 
@@ -587,6 +833,7 @@ def _hermite_cubic(
     #           (------ndm------)
     reps = 4 if len(coord[0])==3 else 2
 
+    from scipy.linalg import block_diag
     # 3x3 rotation into local system
     # Q = rotation(coord, vect)
     # Element length
