@@ -4,11 +4,15 @@
 #
 #===----------------------------------------------------------------------===#
 #
-# Claudio M. Perez
+# Copyright (c) 2025, Claudio M. Perez
+# All rights reserved.  No warranty, explicit or implicit, is provided.
+#
+# This source code is licensed under the BSD 2-Clause License.
+# See LICENSE file or https://opensource.org/licenses/BSD-2-Clause
 #
-import sys
 from collections import defaultdict
 from veux.state  import StateSeries, BasicState, GroupSeriesSE3, GroupStateSE3, GroupStateSO3, Rotation
+import warnings
 
 import numpy as np
 
@@ -28,14 +32,17 @@ class Model:
     def make_state(self, state, **kwds):
         # This just needs to return an opaque object that can be passed into the methods below
         return state 
+    
+    def node_marker(self, tag):
+        return self._node_marker
 
-    def node_location(self, tag): ...
+    def node_position(self, tag, **kwds): ...
 
     def node_information(self, tag): ...
 
     def iter_cells_tags(self, filt=None): ...
 
-    def cell_matches(self, tag): ...
+    def cell_matches(self, tag, type): ...
 
     def cell_type(self, tag):       ... # line triangle quadrilateral 
 
@@ -43,6 +50,10 @@ class Model:
 
     def cell_interior(self, tag):   ...
 
+    def cell_quadrature(self, tag):  return []
+
+    def cell_prototypes(self):  return []
+
     def cell_rotation(self, tag, state): raise NotImplementedError
 
     def cell_position(self, tag, state): raise NotImplementedError
@@ -153,6 +164,11 @@ def read_model(filename:str, shift=None, verbose=False)->dict:
             model = csi.create_model(csi.load(f), verbose=verbose)
         return model.asdict()
 
+    elif isinstance(filename, str) and filename.endswith(".inp"):
+        from openbim import inp
+        model = inp.create_model(inp.parser.load(filename), verbose=verbose, mode="visualize")
+        return model.asdict()
+
     elif isinstance(filename, str) and filename.endswith(".vtk"):
         import meshio
         from .plane import PlaneModel
@@ -345,11 +361,14 @@ def read_state(res_file,
 
 
 class FrameModel:
-# rename to SolidModel?
 
-    def __init__(self, sam:dict,
-                 shift = None, rot=None, # Coordinate transformation
-                 frame_outlines=None, 
+    def __init__(self,
+                 sam:dict,
+                 shift = None,
+                 frame_outlines=None,
+                 node_marker=None,
+                 node_marker_scale=1.0,
+                 xmodel=None,
                  **kwds):
 
         # number of dimensions of artist, not model?
@@ -361,6 +380,11 @@ class FrameModel:
         else:
             shift = np.asarray(shift)
 
+
+        self._xmodel = xmodel
+        self._node_marker = node_marker
+        if node_marker_scale is not None and node_marker is not None:
+            self._node_marker.points *= node_marker_scale
         #
         self._data = _from_opensees(sam, shift, R)# output
 
@@ -371,8 +395,19 @@ class FrameModel:
         # Section data
         #
         from .frame._section import SectionGeometry
+
+        # 1) Supplementary data
+        self._frame_sections = {}
+        for name,section in self._data["sections"].items():
+            _add_section_shape(section, self._data["sections"], self._frame_sections, self.ndm)
+        self._frame_sections = {
+            k: SectionGeometry(v) for k,v in self._frame_sections.items() if len(v) > 0
+        }
+        
+
         self._frame_outlines = {}
         if isinstance(frame_outlines, dict):
+
             for key, polygons in frame_outlines.items():
                 if hasattr(polygons, "exterior"):
                     self._frame_outlines[key] = [polygons]
@@ -388,7 +423,10 @@ class FrameModel:
             # TODO: Make this dict of list of sections
             self._frame_outlines = _get_frame_outlines(self)
 
+        # 2) Default extrusion
         self._extrude_default = SectionGeometry(_OUTLINES[kwds.get("extrude_default", "square")])
+
+        # 3) Forced extrusion
         if "extrude_outline" in kwds:
             if hasattr(kwds["extrude_outline"], "exterior"):
                 self._extrude_outline = SectionGeometry(
@@ -404,9 +442,13 @@ class FrameModel:
 
         self._extrude_scale   = kwds.get("extrude_scale",   1.0)
 
+
     def __getitem__(self, key):
         # TODO: Remove this method
         return self._data[key]
+    
+    def node_marker(self, tag):
+        return self._node_marker
 
     def wrap_state(self,
                     state=None,
@@ -555,6 +597,11 @@ class FrameModel:
     def cell_matches(self, tag, type=None)->bool:
         elem = self["assembly"][tag]
 
+        if type == "prism":
+            return _is_frame(elem) and (
+                "prism" in elem["type"].lower()
+                or "elastic" in elem["type"].lower()
+            )
         if type == "frame":
             return _is_frame(elem)
         
@@ -579,6 +626,25 @@ class FrameModel:
             return np.array([ self.node_position(node, state)
                               for node in self["assembly"][tag]["nodes"] ])
 
+    def frame_element(self, tag):
+        """
+        Iterate over all frame elements in the model.
+        """
+        from veux.frame._element import _FrameElement
+        if self.cell_matches(tag, "frame"):
+            if self.ndm == 2:
+                from .frame._section import SectionGeometry
+                return _FrameElement(tag=tag,
+                                    vmodel=self,
+                                    xmodel=self._xmodel,
+                                    section_override=SectionGeometry([[-1,0],[1,0]]))
+            
+            else:
+                return _FrameElement(tag=tag,
+                                     vmodel=self,
+                                     xmodel=self._xmodel,
+                                     section_override=self._extrude_outline
+                )
 
     def cell_exterior(self, tag):
         """
@@ -590,6 +656,9 @@ class FrameModel:
 
         if "frm" in type or "beamcol" in type:
             return self.cell_indices(tag)
+        
+        elif self.cell_matches(tag, "truss"):
+            return self.cell_indices(tag)
 
         elif ("quad" in type or \
               "shell" in type and ("q" in type) or ("mitc" in type)):
@@ -616,6 +685,19 @@ class FrameModel:
 
         return []
 
+    def cell_quadrature(self, tag):
+        if self.cell_matches(tag, "frame"):
+            if self.cell_matches(tag, "prism"):
+                return []
+            rule = self["assembly"][tag].get("integration", None)
+            if rule is None:
+                return []
+            
+            n = len(self["assembly"][tag]["sections"])
+            from shps.gauss import iquad
+            for xi, wt in zip(*iquad(n=n, rule=rule["type"].lower(), bounds=(0, 1))):
+                yield xi, wt
+
     def cell_interpolation(self, tag):
         pass
 
@@ -641,7 +723,7 @@ class FrameModel:
             if len(nodes) in {4,8,9}:
                 return [[nodes[0], nodes[1], nodes[2]],
                         [nodes[2], nodes[3], nodes[0]]]
-        
+
         elif "tetra" in type:
             nodes = self.cell_indices(tag)
             return [[nodes[0], nodes[2], nodes[1]],
@@ -664,17 +746,10 @@ class FrameModel:
 
         return []
 
-    def _section_area(self, tag, i):
-        # TODO
-        return 1
-        try:
-            stag = self["assembly"][tag]["sections"][i]
-            try:
-                return self["sections"][int(stag)]["A"]
-            except:
-                return self["sections"][str(stag)]["A"]
-        except Exception as e:
-            return  1.0
+    def _frame_section(self, tag):
+        if tag is None:
+            return self._extrude_default
+        return self._frame_sections[int(tag)] if int(tag) in self._frame_sections else self._extrude_default
 
     def frame_section(self, tag, coord=None)->"SectionGeometry":
         from .frame._section import SectionGeometry
@@ -687,24 +762,28 @@ class FrameModel:
         if not self.cell_matches(tag, "frame"):
             return None
 
+
         # # Initialize frame outlines
         # if self._frame_outlines is None:
         #     # TODO: Make this dict of list of sections
         #     self._frame_outlines = _get_frame_outlines(self)
 
-        sections = []
-        if self._extrude_outline is not None:
-            sections = [
-                SectionGeometry(self._extrude_outline.exterior()*self._section_area(tag, 0)),
-            ]*2 #*self._extrude_scale
+        # sections = []
+        # if self._extrude_outline is not None:
+        #     sections = [
+        #         SectionGeometry(self._extrude_outline.exterior()*self._section_area(tag, 0))
+        #     ]*2 #*self._extrude_scale
 
-        elif tag in self._frame_outlines:
-            sections = self._frame_outlines[tag]
+        # elif tag in self._frame_outlines:
+        #     sections = self._frame_outlines[tag]
 
-        elif self._extrude_default is not None:
-            sections = [self._extrude_default]*2 #*self._extrude_scale
+        # elif self._extrude_default is not None:
+        #     sections = [self._extrude_default]*2 #*self._extrude_scale
+        elem = self.frame_element(tag)
+        sections = [elem.sample_section(i) for i in elem.simple_samples()]
 
 
+        # We have sections[], now get the section at the given coordinate
         if len(sections) == 0:
             # print(f"Empty sections for {tag}", file=sys.stderr)
             return
@@ -740,7 +819,7 @@ class FrameModel:
         e1  = v1/L
 
         if self.ndm == 2:
-            v2 = np.array([0, 1, 0])
+            v2 = -np.cross(e1, np.array([0, 0, 1]))
 
         if "yvec" in el["trsfm"] and el["trsfm"]["yvec"] is not None:
             v2  = np.array(el["trsfm"]["yvec"])
@@ -759,6 +838,38 @@ class FrameModel:
         return np.stack([e1,e2,e3])
 
 
+class FiberModel(Model):
+    ndm = 2 
+    ndf = 1
+    def __init__(self, fibers, patches=None):
+        self._fibers = fibers
+
+    def iter_cell_tags(self):
+        yield 1
+
+    def cell_exterior(self):
+        return 
+        yield 
+
+    def cell_interior(self):
+        return 
+        yield
+
+    def wrap_state(self, state, **kwds):
+        return state
+
+    def cell_quadrature(self, tag):
+        if tag == 1:
+            for fiber in self._fibers:
+                if "location"  in fiber:
+                    x = fiber["location"]
+                elif "coord" in fiber:
+                    x = fiber["coord"]
+                else:
+                    x = fiber["y"], fiber["z"]
+                yield (*x, 0), fiber["area"]
+
+
 def _from_opensees(sam: dict, shift, R):
     # Process OpenSees JSON format
 
@@ -834,14 +945,15 @@ def _from_opensees(sam: dict, shift, R):
 def collect_outlines(model):
     return _get_frame_outlines(_from_opensees(model, [0, 0, 0], np.eye(3)))
 
-def _add_section_shape(section, sections, outlines):
+
+def _add_section_shape(section, sections, outlines, ndm):
 
     tag = int(section["name"])
     if "section" in section:
         # Treat aggregated sections
         child_tag = int(section["section"])
         if child_tag not in outlines:
-            _add_section_shape(sections[section["section"]], sections, outlines)
+            _add_section_shape(sections[section["section"]], sections, outlines, ndm)
 
         outlines[tag] = outlines[child_tag]
 
@@ -851,22 +963,23 @@ def _add_section_shape(section, sections, outlines):
                       (1, 0))).T
         outlines[tag] = [R@s for s in section["bounding_polygon"]]
 
-    elif "fibers" in section:
+    elif "fibers" in section and ndm > 2:
         try:
             points = np.array([
                 f.get("coord", None) or f["location"] for f in section["fibers"]
             ])
             try:
                 from veux.utility.alpha_shape import alpha_shape
-                alpha = alpha_shape(points, bound_ratio=0.01) #0.03)#0.01)
-                outlines[tag] =  alpha
+                alpha = alpha_shape(points, bound_ratio=0.01) #0.0025) #0.01) #0.03)#0.01)
+                if len(alpha) > 0:
+                    outlines[tag] =  alpha
             except Exception as e: #scipy.spatial._qhull.QhullError as e:
-                import warnings
-                warnings.warn(str(e))
+                warnings.warn("Failed to compute alpha shape")
                 import scipy.spatial
                 outlines[tag] = points[scipy.spatial.ConvexHull(points).vertices]
+
         except Exception as e:
-            warnings.warn(str(e))
+            warnings.warn("Failed to find section shape")
             return
 
 
@@ -874,8 +987,7 @@ def _add_section_shape(section, sections, outlines):
 def _get_frame_outlines(model):
     section_outlines = {}
     for name,section in model["sections"].items():
-        _add_section_shape(section, model["sections"], section_outlines)
-
+        _add_section_shape(section, model["sections"], section_outlines, model.ndm)
 
     # Function to check if list of lists is homogeneous
     homogeneous = lambda lst: (
@@ -889,19 +1001,20 @@ def _get_frame_outlines(model):
     outlines = {}
     for elem in model["assembly"].values():
         if "sections" in elem:
-            outlines[elem["name"]] = [
+            elem_shapes = [
                 SectionGeometry(section_outlines[int(i)]) for i in elem["sections"]
                 if i in section_outlines and section_outlines[i] is not None
             ]
-            continue
-            if len(elem_shapes) == 0:
-                continue
-            if not homogeneous(elem_shapes):
-                elem_shapes = np.array(elem_shapes[0])
-            else:
-                elem_shapes = np.array(elem_shapes)
+            # continue
+            if len(elem_shapes) != 0:
+                outlines[elem["name"]] = elem_shapes
 
-            outlines[elem["name"]] = [SectionGeometry(shape) for shape in elem_shapes]
+            # if not homogeneous(elem_shapes):
+            #     elem_shapes = np.array(elem_shapes[0])
+            # else:
+            #     elem_shapes = np.array(elem_shapes)
+
+            # outlines[elem["name"]] = [SectionGeometry(shape) for shape in elem_shapes]
 
 
     return outlines
