@@ -1,6 +1,6 @@
 import numpy as np
-import matplotlib.pyplot as plt
-import matplotlib.tri as tri
+# import matplotlib.pyplot as plt
+# import matplotlib.tri as tri
 
 from .elements import (
     quads_to_4tris,
@@ -213,9 +213,9 @@ def node_average(model, response, ndm=2, keys=None):
 
     Returns:
         dict[node_tag] = {
-            "xx": float, 
+            "xx": float,
             "yy": float,
-            ... 
+            ...
         }
     """
 
@@ -227,13 +227,7 @@ def node_average(model, response, ndm=2, keys=None):
     #
     # Below, we simply re-use the user's helper for 2D, but you'll have to define
     # your own for 3D or unify them if your model can contain both:
-    ele_tags_all = model.getEleTags()
-    if ndm == 2:
-        ele_tags = _stress_2d_ele_tags_only(model, ele_tags_all)
-    else:
-        # Placeholder; you must implement a 3D filter or skip if you already know
-        # the elements are 3D:
-        ele_tags = ele_tags_all
+    ele_tags = model.getEleTags()
 
     # For a typical 2D element with 3 stress components: sxx, syy, sxy
     # For a typical 3D element with 6 stress components: sxx, syy, szz, sxy, syz, sxz
